
#include "DetectorConstruction.hh"

#include "G4NistManager.hh"
#include "G4Box.hh"
#include "G4Tubs.hh"
#include "G4LogicalVolume.hh"
#include "G4PVPlacement.hh"
#include "G4PVDivision.hh"

#include "G4SDManager.hh"
#include "G4MultiFunctionalDetector.hh"
#include "G4VPrimitiveScorer.hh"
#include "G4PSCellFlux.hh"

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

DetectorConstruction::DetectorConstruction()
{
  nbRings     = 3;
  drRing      = 1*mm;

  bagRadius   = nbRings*drRing;  
  bagSizeZ    = 2*bagRadius;
  worldSizeXY = 2.4*bagRadius;
  worldSizeZ  = 1.2*bagSizeZ;    
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

DetectorConstruction::~DetectorConstruction()
{}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

G4VPhysicalVolume* DetectorConstruction::Construct()
{
  //
  // define a material
  //   
  G4Material* Air =
  G4NistManager::Instance()->FindOrBuildMaterial("G4_AIR"); 
  
  //     
  // World
  //
  G4Box*  
  solidWorld = new G4Box("World",				//its name
                   worldSizeXY/2,worldSizeXY/2,worldSizeZ/2);	//its size
		   
  G4LogicalVolume*                         
  logicWorld = new G4LogicalVolume(solidWorld,		//its solid
                                   Air,			//its material
                                   "World");		//its name
  G4VPhysicalVolume*                                   
  physiWorld = new G4PVPlacement(0,			//no rotation
  				 G4ThreeVector(),	//at (0,0,0)
                                 logicWorld,		//its logical volume
                                 "World",		//its name
                                 0,			//its mother  volume
                                 false,			//no boolean operation
                                 0);			//copy number
				 
  //     
  // Bag
  //
  G4Tubs*  
  solidBag = new G4Tubs("Bag",				//its name
            0., bagRadius, 0.5*bagSizeZ, 0., twopi);	//its size
		   
  G4LogicalVolume*                         
  logicBag = new G4LogicalVolume(solidBag,		//its solid
                                   Air,			//its material
                                   "Bag");		//its name
				   
             new G4PVPlacement(0,			//no rotation
  			       G4ThreeVector(),		//at (0,0,0)
                               logicBag,		//its logical volume
                               "Bag",			//its name
                               logicWorld,		//its mother  volume
                               false,			//no boolean operation
                               0);			//copy number				     
  //     
  // Ring
  //
  G4Tubs*  
  solidRing = new G4Tubs("Ring",			//its name
            0., drRing, 0.5*bagSizeZ, 0., twopi);	//its size
		   
  G4LogicalVolume*                         
  logicRing = new G4LogicalVolume(solidRing,		//its solid
                                   Air,			//its material
                                   "Ring");		//its name
				   
               new G4PVDivision("Ring",			//its name
      			       logicRing,		//its logical volume
      			       logicBag,		//its mother
                               kRho,			//axis of replication
                               nbRings,			//number of replica
                               0.);			//offset
			       				 
  // declare Ring as scorer
  //
  G4SDManager* SDman = G4SDManager::GetSDMpointer();
  SDman->SetVerboseLevel(2);
  
  G4MultiFunctionalDetector* det = new G4MultiFunctionalDetector("myDetector");
  det->RegisterPrimitive(new G4PSCellFlux("cellFlux"));
  SDman->AddNewDetector(det);
  logicRing->SetSensitiveDetector(det);
       	 
  //
  //always return the physical World
  //  
  return physiWorld;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
